@@ -49,20 +49,18 @@ def get_app_dir(app_dir=None):
         return os.path.realpath(app_dir)
 
     # Use the default locations for data_files.
+    app_dir = pjoin(sys.prefix, 'share', 'jupyter', 'lab')
 
     # Check for a user level install.
     if hasattr(site, 'getuserbase') and here.startswith(site.getuserbase()):
         app_dir = pjoin(site.getuserbase(), 'share', 'jupyter', 'lab')
 
     # Check for a system install in '/usr/local/share'.
-    elif (sys.prefix == '/usr' and
+    elif (sys.prefix.startswith('/usr') and not
+          os.path.exists(app_dir) and
           os.path.exists('/usr/local/share/jupyter/lab')):
         app_dir = '/usr/local/share/jupyter/lab'
 
-    # Otherwise use the sys prefix.
-    else:
-        app_dir = pjoin(sys.prefix, 'share', 'jupyter', 'lab')
-
     return os.path.realpath(app_dir)
 
 
@@ -154,6 +152,12 @@ def install_extension_async(extension, app_dir=None, logger=None, abort_callback
             _write_build_config(config, app_dir, logger=logger)
         return
 
+    # Run npm install if needed
+    if (os.path.exists(extension) and
+            not os.path.exists(pjoin(extension, 'node_modules'))):
+        yield run([get_npm_name(), 'install'], cwd=extension, logger=logger,
+                  abort_callback=abort_callback)
+
     _ensure_app_dirs(app_dir, logger)
 
     target = pjoin(app_dir, 'extensions', 'temp')
@@ -162,7 +166,8 @@ def install_extension_async(extension, app_dir=None, logger=None, abort_callback
     os.makedirs(target)
 
     # npm pack the extension
-    yield run([get_npm_name(), 'pack', extension], cwd=target, logger=logger, abort_callback=abort_callback)
+    yield run([get_npm_name(), 'pack', extension], cwd=target, logger=logger,
+              abort_callback=abort_callback)
 
     fnames = glob.glob(pjoin(target, '*.*'))
     if not fnames:
@@ -204,12 +209,6 @@ def install_extension_async(extension, app_dir=None, logger=None, abort_callback
     shutil.move(pjoin(target, fname), pjoin(app_dir, 'extensions'))
     shutil.rmtree(target)
 
-    # Clear staging/node_modules
-    target = pjoin(app_dir, 'staging', 'node_modules')
-    target = target.replace('/', os.sep)
-    if os.path.exists(target):
-        shutil.rmtree(target)
-
 
 def link_package(path, app_dir=None, logger=None):
     """Link a package against the JupyterLab build."""
@@ -599,7 +598,7 @@ def build_async(app_dir=None, name=None, version=None, logger=None, abort_callba
     npm = get_npm_name()
 
     # Make sure packages are installed.
-    yield run([npm, 'install', '--no-optional'], cwd=staging, logger=logger, abort_callback=abort_callback)
+    yield run([npm, 'install'], cwd=staging, logger=logger, abort_callback=abort_callback)
 
     # Build the app.
     yield run([npm, 'run', 'clean'], cwd=staging, logger=logger, abort_callback= abort_callback)
@@ -832,7 +831,7 @@ def _ensure_package(app_dir, logger=None, name=None, version=None):
             shutil.rmtree(staging)
             os.makedirs(staging)
 
-    for fname in ['index.app.js', 'webpack.config.js']:
+    for fname in ['index.app.js', 'webpack.config.js', '.npmrc']:
         dest = pjoin(staging, fname.replace('.app', ''))
         shutil.copy(pjoin(here, fname), dest)
 
